@@ -38,16 +38,19 @@ def register(register_settings, package):
     repository.close()
 
     if resp.is_redirect:
-        raise exceptions.RedirectDetected(
-            ('"{0}" attempted to redirect to "{1}" during registration.'
-             ' Aborting...').format(repository_url,
-                                    resp.headers["location"]))
+        raise exceptions.RedirectDetected.from_args(
+            repository_url,
+            resp.headers["location"],
+        )
 
     resp.raise_for_status()
 
 
 def main(args):
-    parser = argparse.ArgumentParser(prog="twine register")
+    parser = argparse.ArgumentParser(
+        prog="twine register",
+        description="register operation is not required with PyPI.org",
+    )
     settings.Settings.register_argparse_arguments(parser)
     parser.add_argument(
         "package",
